﻿using Bharabas.Data;
using Bharabas.Models;
using Microsoft.AspNetCore.Mvc;

namespace Bharabas.Controllers
{
    public class AudioController : Controller
    {
        private readonly ApplicationDbContext _db;

        public AudioController (ApplicationDbContext db)
        {
            _db = db;
        }

        public IActionResult Index()
        {
            IEnumerable<Audio> objAudioList = _db.AudioList.ToList();
            this.ViewBag.listCategory = _db.Category.ToList();
            return View(objAudioList);
        }

        /* 
         * GET SELECTED AUDIO
         */
        [HttpGet]
        public IActionResult AudioDetail(int? id)
        {
			if (id == null || id == 0)
			{
				return NotFound();
			}
			var selectedAudio = _db.AudioList.Find(id);
			if (selectedAudio == null)
			{
				return NotFound();
			}
            var selectedAudioCategoryIds = _db.AudioCategoryRelation.Where(u => u.audioId == id).Select(u => u.categoryId).ToList();
            var selectedAudioCategories = new List<Category>();
            foreach (int i in selectedAudioCategoryIds)
            {
                var category = _db.Category.Find(i);
                if (category != null)
                {
                    selectedAudioCategories.Add(category);
                    Console.WriteLine(i+ ": " + category.name);
                } else
                {
                    Console.WriteLine(i.ToString());
                }
            }
            var selectedAudioComment = _db.CommentLog.Where(u => u.audioId == id).ToList();
            this.ViewBag.listCategory = _db.Category.ToList();
            return View(Tuple.Create(selectedAudio, selectedAudioCategories, selectedAudioComment));
        }


        /*
         * INSERT TO DATABASE
         */
        public void InsertAudio(Audio objAudio, Category objCategory)
        {
            _db.AudioList.Add(objAudio);
            _db.SaveChanges();
            var audioId = _db.AudioList.Where(u => u.title == objAudio.title).Select(u => u.audioId).ToList();
            var categoryFound = _db.Category.Where(u => u.name == objCategory.name).Select(u => u.categoryId).ToList();
            if (categoryFound.Count == 0)
            {
                _db.Category.Add(objCategory);
                _db.SaveChanges();
            }
            var categoryId = _db.Category.Where(u => u.name == objCategory.name).Select(u => u.categoryId).ToList();
            AudioCategoryRelation relation = new AudioCategoryRelation();
            relation.audioId = audioId[0];
            relation.categoryId = categoryId[0];
            _db.AudioCategoryRelation.Add(relation);
            _db.SaveChanges();
            TempData["success"] = "New Audio record inserted successfully";
        }
        //GET 
        public IActionResult Create()
        {
            return View();
        }
        //POST
        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Create(Audio objAudio, Category objCategory)
        {
            if (ModelState.IsValid)
            {
                InsertAudio(objAudio, objCategory);
            }
            return View(objAudio);
        }


        /*
         * UPDATE DATABASE
         */
        public void UpdateAudio(Audio objAudio)
        {
            _db.AudioList.Update(objAudio);
            _db.SaveChanges();
            TempData["success"] = "Audio record updated successfully";
        }
        //GET
        public IActionResult AudioDetailEdit(int? id)
        {
            if (id == null || id == 0)
            {
                return NotFound();
            }
            var selectedAudio = _db.AudioList.Find(id);
			if (selectedAudio == null) {
                return NotFound();
            }
			var selectedAudioCategoryIds = _db.AudioCategoryRelation.Where(u => u.audioId == id).Select(u => u.categoryId).ToList();
			var selectedAudioCategories = new List<Category>();
			foreach (int i in selectedAudioCategoryIds)
			{
				var category = _db.Category.Find(i);
				if (category != null)
				{
					selectedAudioCategories.Add(category);
					Console.WriteLine(i + ": " + category.name);
				}
				else
				{
					Console.WriteLine(i.ToString());
				}
			}
			var selectedAudioComment = _db.CommentLog.Where(u => u.audioId == id).ToList();
            var allCategory = _db.Category.ToList();
			return View(Tuple.Create(selectedAudio, selectedAudioCategories, selectedAudioComment, allCategory));
        }
        //POST
        public IActionResult Edit(Audio objAudio)
        {
            if (ModelState.IsValid)
            {
                UpdateAudio(objAudio);
            }
            return View(objAudio);
        }


        /* 
         * DELETE FROM DATABASE
         */
        public void DeleteAudio(Audio objAudio)
        {
            _db.AudioList.Remove(objAudio);
            var relation = _db.AudioCategoryRelation.Where(u => u.audioId == objAudio.audioId).ToList();
            var comments = _db.CommentLog.Where(u => u.audioId == objAudio.audioId).ToList();
            var likes = _db.LikeLog.Where(u => u.audioId == objAudio.audioId).ToList();
            var share = _db.ShareLog.Where(u => u.audioId == objAudio.audioId).ToList();            
            for (int i = 0; i < relation.Count; i++)
            {
                _db.AudioCategoryRelation.Remove(relation[i]);
                _db.CommentLog.Remove(comments[i]);
                _db.LikeLog.Remove(likes[i]);
                _db.ShareLog.Remove(share[i]);
            }
            _db.SaveChanges();
            TempData["success"] = "Audio record deleted successfully";
        }
        //GET
        public IActionResult Delete(int? id)
        {
            if (id == null || id == 0)
            {
                return NotFound();
            }
            var recordFound = _db.AudioList.Find(id);
            if (recordFound == null)
            {
                return NotFound();
            }
            return View(recordFound);
        }
        //POST
        public IActionResult Delete(Audio objAudio)
        {
            if (ModelState.IsValid)
            {
                DeleteAudio(objAudio);
            }
            return View(objAudio);
        }
    }
}
