﻿using Bharabas.Data;
using Bharabas.Models;
using Microsoft.AspNetCore.Mvc;

namespace Bharabas.Controllers
{
	public class CommentController : Controller
	{
		private readonly ApplicationDbContext _db;

		public CommentController(ApplicationDbContext db)
		{
			_db = db;
		}

		public IActionResult Index()
		{
			IEnumerable<CommentLog> objCommentList = _db.CommentLog.ToList();
			return View(objCommentList);
		}

		public IActionResult SelectedAudioComment(int audioId)
		{
			var selectedAudioComment = _db.CommentLog.Where(u => u.audioId == audioId).ToList();
			return PartialView(selectedAudioComment);
		}

		/* ADD COMMENT */
		[HttpPost]
		[ValidateAntiForgeryToken]
		public IActionResult AddComment()
		{
			int audioId = 0;
			int referedCommentId = 0;
			int.TryParse(HttpContext.Request.Form["audio-id"], out audioId);
			int.TryParse(HttpContext.Request.Form["refered-comment-id"], out referedCommentId);
			var commentText = HttpContext.Request.Form["comment-box"].ToString();
			CommentLog comment = new CommentLog();
			Console.WriteLine($"{audioId} {commentText}");
			if (ModelState.IsValid)
			{
				comment.audioId = audioId;
				comment.referedCommentId = referedCommentId;
				comment.commentText = commentText;
				comment.createdOn = DateTime.Now;
				_db.CommentLog.Add(comment);
				_db.SaveChanges();
				TempData["success"] = "New Comment inserted successfully";
			}
			ViewBag.Message = "Comment have been posted";
			return RedirectToAction("AudioDetail", "Audio", new { id = audioId });
		}

		/* EDIT COMMENT */
		//GET
		public IActionResult EditComment(int? commentId)
		{
			if (commentId == null || commentId == 0)
			{
				return NotFound();
			}
			var recordFound = _db.CommentLog.Find(commentId);
			if (recordFound == null)
			{
				return NotFound();
			}
			return View(recordFound);
		}
		//POST
		public IActionResult EditComment(CommentLog comment)
		{
			if (ModelState.IsValid)
			{
				_db.CommentLog.Update(comment);
				_db.SaveChanges();
				TempData["success"] = "Comment record updated successfully";
			}
			return View(comment);
		}

		/* DELETE COMMENT */
		//GET
		public IActionResult Delete(int? commentId)
		{
			if (commentId == null || commentId == 0)
			{
				return NotFound();
			}
			var recordFound = _db.CommentLog.Find(commentId);
			if (recordFound == null)
			{
				return NotFound();
			}
			return View(recordFound);
		}
		//POST
		public IActionResult Delete(CommentLog comment)
		{
			if (ModelState.IsValid)
			{
				_db.CommentLog.Remove(comment);
				_db.SaveChanges(true);
				TempData["success"] = "Comment record deleted successfully";
			}
			return View(comment);
		}
	}
}
