﻿using Bharabas.Data;
using Bharabas.Models;
using Bharabas.Scraping;
using Hangfire;
using Hangfire.Storage;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System.Collections;
using System.Globalization;
using System.Text.Encodings.Web;

namespace Bharabas.Controllers
{
	[Authorize]
    public class DashboardController : Controller
    {
        private readonly ApplicationDbContext _db;
        const int pageSize = 10;

        public DashboardController(ApplicationDbContext db)
        {
            _db = db;
        }
        public IActionResult Index(int pg = 1, string q = "")
        {
            if(q == null)
            {
                q = ""; 
            }
            int cnt = _db.AudioList.Count();
            if(pg > cnt)
            {
                pg = 1;
            }
            var pager =  new Pager(cnt, pg,pageSize);

            ViewBag.pager = pager;
            ViewBag.q = "";
            ViewBag.listAudio = _db.AudioList.Where(u=>u.title.ToLower().Contains(q.ToLower())).Skip((pg-1)*pageSize).Take(10).ToList();
            return View();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Delete(int id)
        {
            var toDeleteAudio = _db.AudioList.Where(u => u.audioId == id).First();
            var toDeleteAudioCategory = _db.AudioCategoryRelation.Where(u => u.audioId == id).ToList();
            foreach(var item in toDeleteAudioCategory)
            {
                _db.AudioCategoryRelation.Remove(item);
            }
            _db.AudioList.Remove(toDeleteAudio);
            _db.SaveChanges();
            
            return RedirectToAction("", "Dashboard");
        }

		/*
         * UPDATE DATABASE
         */
		public void UpdateAudio(Audio objAudio, String[] objRelationList) 
		{
			_db.AudioList.Update(objAudio);
			var relation = _db.AudioCategoryRelation.Where(u => u.audioId == objAudio.audioId).ToList();
			for (int i = 0; i < relation.Count; i++)
			{
				_db.AudioCategoryRelation.Remove(relation[i]);
			}
			foreach (var item in objRelationList)
			{
				AudioCategoryRelation rel = new AudioCategoryRelation();
				rel.audioId = objAudio.audioId;
				rel.categoryId = Int32.Parse(item);
				Console.WriteLine(rel);
				Console.WriteLine(rel.audioId);
				Console.WriteLine(rel.categoryId);
				_db.AudioCategoryRelation.Add(rel);
			}
			_db.SaveChanges();
			TempData["success"] = "Audio record updated successfully";
		}
		//GET
		public IActionResult Edit(int? id)
		{
			if (id == null || id == 0)
			{
				return NotFound();
			}
			var selectedAudio = _db.AudioList.Find(id);
			if (selectedAudio == null)
			{
				return NotFound();
			}
			var selectedAudioCategoryIds = _db.AudioCategoryRelation.Where(u => u.audioId == id).Select(u => u.categoryId).ToList();
			var selectedAudioCategories = new List<Category>();
			foreach (int i in selectedAudioCategoryIds)
			{
				var category = _db.Category.Find(i);
				if (category != null)
				{
					selectedAudioCategories.Add(category);
					Console.WriteLine(i + ": " + category.name);
				}
				else
				{
					Console.WriteLine(i.ToString());
				}
			}
			var selectedAudioComment = _db.CommentLog.Where(u => u.audioId == id).ToList();
			var allCategory = _db.Category.ToList();
			return View(Tuple.Create(selectedAudio, selectedAudioCategories, selectedAudioComment, allCategory));
		}

        public IActionResult Add()
        {
            ViewBag.listCategory = _db.Category.ToList();
            return View();
        }

		[HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult SubmitEdit()
        {
			int audioId = 0;
			int.TryParse(HttpContext.Request.Form["audio-id"], out audioId);
			float duration = 0;
			float.TryParse(HttpContext.Request.Form["duration"], out duration);
			var imageSource = HttpContext.Request.Form["image-source"].ToString();
			var audioSource = HttpContext.Request.Form["audio-source"].ToString();
			var titleSong = HttpContext.Request.Form["title-song"].ToString();
			var artistSong = HttpContext.Request.Form["content-creator"].ToString();
			var dateUpload = DateOnly.FromDateTime(DateTime.Now);
			DateOnly.TryParse(HttpContext.Request.Form["date-uploaded"], out dateUpload);
			var platformLink = HttpContext.Request.Form["platform-source"].ToString();
			var platform = HttpContext.Request.Form["platform-selected"].ToString();
			var description = HttpContext.Request.Form["description"].ToString();
			var categoryIds = HttpContext.Request.Form["category-selector"];

			Console.WriteLine("1: " + audioId);
			Console.WriteLine("2: " + imageSource);
			Console.WriteLine("3: " + audioSource);
			Console.WriteLine("3: " + titleSong);
			Console.WriteLine("4: " + artistSong);
			Console.WriteLine("5: " + dateUpload);
			Console.WriteLine("6: " + platformLink);
			Console.WriteLine("7: " + platform);
			Console.WriteLine("8: " + description);
			Console.WriteLine("9: " + categoryIds);

			if (imageSource == "" || audioSource == "" || titleSong == "" || artistSong == "" || dateUpload.Equals(null) || platformLink == "" || platform == "" || description == "")
			{
				return Content("<script language='javascript' type='text/javascript'>alert('Input cannot be empty!'); window.history.back();</script>", "text/html");
			}
			
			else
			{
				Audio updatedAudio = new Audio();
				updatedAudio.audioId = audioId;
				updatedAudio.duration = (int)duration;
				updatedAudio.imageUrl = imageSource;
				updatedAudio.url = audioSource;
				updatedAudio.title = titleSong;
				updatedAudio.contentCreator = artistSong;
				updatedAudio.dateUploaded = dateUpload;
				updatedAudio.platformUrl = platformLink;
				updatedAudio.platform = platform;
				updatedAudio.description = description;
				UpdateAudio(updatedAudio, categoryIds);
			}
			return RedirectToAction("Edit", "Dashboard", new { id = audioId });
        }


        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult DeleteComment(int id)
        {
			Console.WriteLine(id);
			var toDeleteComment = _db.CommentLog.Where(u => u.commentId == id).First();
			Console.WriteLine(toDeleteComment);
			_db.CommentLog.Remove(toDeleteComment);
            _db.SaveChanges();
			
			var toDeleteChildComment = _db.CommentLog.Where(u => u.referedCommentId == id);
			Console.WriteLine(toDeleteChildComment);
			foreach (var comment in toDeleteChildComment)
			{
				Console.WriteLine(comment);
				_db.CommentLog.Remove(comment);
			}
			_db.SaveChanges();
            return RedirectToAction("Edit", "Dashboard", new { id = toDeleteComment.audioId });
        }


        [HttpPost]
        public IActionResult SubmitAdd()
        {
            Console.WriteLine("CEK 123");
            var imageSource = Request.Form["imageSource"];
			var audioSource = Request.Form["audioSource"];
			var titleSong = Request.Form["titleSong"];
			var artistSong = Request.Form["artistSong"];
			var dateUpload = Request.Form["dateUpload"];
			var platformLink = Request.Form["platformLink"];
			var platform = Request.Form["platform"];
			var description = Request.Form["description"];
            float duration = 0;
            float.TryParse(HttpContext.Request.Form["duration"], out duration);
            var categoryIds = HttpContext.Request.Form["category-selector"];

			Console.WriteLine("Category list");
			foreach(var id in categoryIds)
			{
				Console.WriteLine(id);
			}


			if (imageSource == "" || audioSource == "" || titleSong == "" || artistSong == "" || dateUpload == "" || platform == "" || description == "")
            {
				return Content("<script language='javascript' type='text/javascript'>alert('Input cannot be empty!'); window.history.back();</script>", "text/html");
            }

			var isDuplicate = _db.AudioList.Where(u => u.url == (string)audioSource).Select(u => u.audioId).ToList().Count () > 0;
			if (isDuplicate)
			{
				return Content("<script language='javascript' type='text/javascript'>alert('Audio content already exists!'); window.history.back();</script>", "text/html");
			}


			var newObjAudio = new Audio();
            newObjAudio.title = titleSong;
            newObjAudio.description = description;
            newObjAudio.url = audioSource;
            newObjAudio.imageUrl = imageSource;
            newObjAudio.platform = platform;
            newObjAudio.platformUrl = platformLink;
            newObjAudio.contentCreator = artistSong;
            newObjAudio.dateUploaded = DateOnly.Parse(dateUpload);
            newObjAudio.duration = (int)duration;

            _db.AudioList.Add(newObjAudio);
            _db.SaveChanges();
            Console.WriteLine("Tambah audio");

			var audioId = _db.AudioList.Where(u => u.url == (string)audioSource).Select(u => u.audioId).ToList();

			newObjAudio.audioId = audioId[0];

			UpdateAudio(newObjAudio, categoryIds);

			Console.WriteLine("Tambah category");
			return RedirectToAction("", "Dashboard");
        }

		[HttpPost]
		public IActionResult SubmitCategory()
        {
			var categoryName = Request.Form["categoryName"];
			if (string.IsNullOrWhiteSpace(categoryName))
			{
                return Content("<script language='javascript' type='text/javascript'>alert('Input cannot be empty!'); window.history.back();</script>", "text/html");
            }
            var isDuplicate = _db.Category.Where(u => u.name.Replace(" ", "").ToLower() == ((string)categoryName).Replace(" ", "").ToLower()).Select(u => u.categoryId).ToList().Count() > 0;
            if (isDuplicate)
            {
                return Content("<script language='javascript' type='text/javascript'>alert('Category already exists!'); window.history.back();</script>", "text/html");
            }
            var newObjCategory = new Category();

			newObjCategory.name = categoryName;
			_db.Category.Add(newObjCategory);
			_db.SaveChanges();
			Console.WriteLine("Tambah category");

			return RedirectToAction("", "Dashboard");

		}

        public IActionResult ScrapeAudio()
		{
			var cronExpression = JobStorage.Current.GetConnection().GetRecurringJobs()[0].Cron;
			var tokens = cronExpression.Split(' ');
			var minute = getSuffixNumber(tokens[0]);
			var hour = getSuffixNumber(tokens[1]);
			var day = getSuffixNumber(tokens[2]);
			Console.WriteLine($"{day} day {hour} hour {minute} minute");
			return View(Tuple.Create(day,hour,minute));
		}

		private int getSuffixNumber(string v)
		{
			int res = 0;
			for(int i=v.Length-1; i>=0; i--)
			{
				if (v[i] < '0' || v[i] > '9')
				{
					break;
				}
				else
				{
					res += (int)Math.Pow(10, v.Length - 1 - i) * (v[i] - '0');
				}
			}
			return res;
		}

		public IActionResult ScrapePlatform(string platform)
		{
			Console.WriteLine(platform);
			if (platform == "ListenNotes")
			{
				BackgroundJob.Enqueue<ScrapingListenNotes>(x => x.scrapeData());
			}
			else if (platform == "Google Podcast")
			{
				BackgroundJob.Enqueue<ScrapingGooglePodcast>(x => x.scrapeDataAsync());
			}
			else if (platform == "YouTube")
			{
				BackgroundJob.Enqueue<ScrapingYouTube>(x => x.ScrapeData());
			}
			else if (platform == "Spotify")
			{
				BackgroundJob.Enqueue<ScrapingAnchor>(x => x.scrapeDataAsync());
			}
			return Redirect("/Dashboard/Hangfire/jobs/processing");
		} 

		public IActionResult EditScrapeTimer()
		{
			var day = int.Parse(Request.Form["day"]!);
			var hour = int.Parse(Request.Form["hour"]!);
			var minute = int.Parse(Request.Form["minute"]!);
			string cronExpression = $"{(minute > 0 ? "*/" + minute.ToString() : "0")} {(hour > 0 ? "*/" + hour.ToString() : "0")} {(day > 0 ? "*/" + day.ToString() : "0")} * *";
			RecurringJob.AddOrUpdate<ScrapingListenNotes>("Periodic Scrape Listen Notes", x => x.scrapeData(), cronExpression, TimeZoneInfo.FindSystemTimeZoneById("SE Asia Standard Time"));
			RecurringJob.AddOrUpdate<ScrapingGooglePodcast>("Periodic Scrape Google Podcast", x => x.scrapeDataAsync(), cronExpression, TimeZoneInfo.FindSystemTimeZoneById("SE Asia Standard Time"));
			RecurringJob.AddOrUpdate<ScrapingYouTube>("Periodic Scrape YouTube", x => x.ScrapeData(), cronExpression, TimeZoneInfo.FindSystemTimeZoneById("SE Asia Standard Time"));
			RecurringJob.AddOrUpdate<ScrapingAnchor>("Periodic Scrape Spotify", x => x.scrapeDataAsync(), cronExpression, TimeZoneInfo.FindSystemTimeZoneById("SE Asia Standard Time"));
			return Redirect("/Dashboard/Hangfire/recurring");
		}

		public IActionResult ScrapeRSS()
		{
			string rssUrl = Request.Form["rssUrl"]!;
			Console.WriteLine($"rss url: {rssUrl}");
			BackgroundJob.Enqueue<ScrapingRSS>(x => x.scrapeRSSAsync(rssUrl));
			return Redirect("/Dashboard/Hangfire/jobs/processing");
		}
    }
}
