﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;
using Npgsql.EntityFrameworkCore.PostgreSQL.Metadata;

#nullable disable

namespace Bharabas.Migrations
{
    /// <inheritdoc />
    public partial class bharabasdbV1 : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateTable(
                name: "AudioList",
                columns: table => new
                {
                    audioId = table.Column<int>(type: "integer", nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    title = table.Column<string>(type: "text", nullable: false),
                    description = table.Column<string>(type: "text", nullable: false),
                    url = table.Column<string>(type: "text", nullable: false),
                    contentCreator = table.Column<string>(type: "text", nullable: false),
                    platform = table.Column<string>(type: "text", nullable: false),
                    duration = table.Column<int>(type: "integer", nullable: false),
                    lastUpdated = table.Column<DateTime>(type: "timestamp with time zone", nullable: false),
                    likesCount = table.Column<int>(type: "integer", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_AudioList", x => x.audioId);
                });

            migrationBuilder.CreateTable(
                name: "Category",
                columns: table => new
                {
                    categoryId = table.Column<int>(type: "integer", nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    name = table.Column<string>(type: "text", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_Category", x => x.categoryId);
                });

            migrationBuilder.CreateTable(
                name: "CommentLog",
                columns: table => new
                {
                    commentId = table.Column<int>(type: "integer", nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    createdOn = table.Column<DateTime>(type: "timestamp with time zone", nullable: false),
                    audioId = table.Column<int>(type: "integer", nullable: false),
                    referedCommentId = table.Column<int>(type: "integer", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_CommentLog", x => x.commentId);
                    table.ForeignKey(
                        name: "FK_CommentLog_AudioList_audioId",
                        column: x => x.audioId,
                        principalTable: "AudioList",
                        principalColumn: "audioId",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_CommentLog_CommentLog_referedCommentId",
                        column: x => x.referedCommentId,
                        principalTable: "CommentLog",
                        principalColumn: "commentId",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "LikeLog",
                columns: table => new
                {
                    likeId = table.Column<int>(type: "integer", nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    createdOn = table.Column<DateTime>(type: "timestamp with time zone", nullable: false),
                    audioId = table.Column<int>(type: "integer", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_LikeLog", x => x.likeId);
                    table.ForeignKey(
                        name: "FK_LikeLog_AudioList_audioId",
                        column: x => x.audioId,
                        principalTable: "AudioList",
                        principalColumn: "audioId",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "ShareLog",
                columns: table => new
                {
                    shareId = table.Column<int>(type: "integer", nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    createdOn = table.Column<DateTime>(type: "timestamp with time zone", nullable: false),
                    audioId = table.Column<int>(type: "integer", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_ShareLog", x => x.shareId);
                    table.ForeignKey(
                        name: "FK_ShareLog_AudioList_audioId",
                        column: x => x.audioId,
                        principalTable: "AudioList",
                        principalColumn: "audioId",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "AudioCategoryRelation",
                columns: table => new
                {
                    audioId = table.Column<int>(type: "integer", nullable: false),
                    categoryId = table.Column<int>(type: "integer", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_AudioCategoryRelation", x => new { x.audioId, x.categoryId });
                    table.ForeignKey(
                        name: "FK_AudioCategoryRelation_AudioList_audioId",
                        column: x => x.audioId,
                        principalTable: "AudioList",
                        principalColumn: "audioId",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_AudioCategoryRelation_Category_categoryId",
                        column: x => x.categoryId,
                        principalTable: "Category",
                        principalColumn: "categoryId",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "IX_AudioCategoryRelation_categoryId",
                table: "AudioCategoryRelation",
                column: "categoryId");

            migrationBuilder.CreateIndex(
                name: "IX_CommentLog_audioId",
                table: "CommentLog",
                column: "audioId");

            migrationBuilder.CreateIndex(
                name: "IX_CommentLog_referedCommentId",
                table: "CommentLog",
                column: "referedCommentId");

            migrationBuilder.CreateIndex(
                name: "IX_LikeLog_audioId",
                table: "LikeLog",
                column: "audioId");

            migrationBuilder.CreateIndex(
                name: "IX_ShareLog_audioId",
                table: "ShareLog",
                column: "audioId");
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "AudioCategoryRelation");

            migrationBuilder.DropTable(
                name: "CommentLog");

            migrationBuilder.DropTable(
                name: "LikeLog");

            migrationBuilder.DropTable(
                name: "ShareLog");

            migrationBuilder.DropTable(
                name: "Category");

            migrationBuilder.DropTable(
                name: "AudioList");
        }
    }
}
