﻿using Bharabas.Data;
using Bharabas.Models;
using Bharabas.Scraping;
using Google.Apis.CustomSearchAPI.v1;
using Google.Apis.Services;
using HtmlAgilityPack;
using SpotifyAPI.Web;
using SpotifyAPI.Web.Http;
using System.ServiceModel.Syndication;
using System.Text.Json.Nodes;
using System.Xml;

namespace Bharabas.Scraping
{
    public class ScrapingAnchor
    {
        const string chartPodcastIDUrl = "https://podcastcharts.byspotify.com/api/charts/top?region=id";
        const string chartEpsIDUrl = "https://podcastcharts.byspotify.com/api/charts/top_episodes?region=id";
        char[] c = new char[] { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
        readonly string[] allowedLanguages = new string[] {"en", "in", "id"};
        int count = 0;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly ApplicationDbContext _db;
        private readonly IConfiguration _config;
        private static string spotifyAccessToken = "dummy";
        public ScrapingAnchor(ApplicationDbContext db, IHttpClientFactory httpClientFactory, IConfiguration config)
        {
            _httpClientFactory= httpClientFactory;
            _db = db;
            _config = config;
		}

		public async Task scrapeDataAsync()
        {
            // kumpulin id
            //await getRssIndoFromChart();

            // brute force 7-8 char
            // baru 0 - 36^3-1
            //long start = (long)36*36*36;
            //long end = (long)36 * 36 * 36*36 * 36 * 36 * 36;
            //for(long i=start;i<end;i++)
            //{
            //    //Console.WriteLine("Count = "+count.ToString());
            //    //if (count > 0) break;
            //    string rssId = "";
            //    long id = i;
            //    for(int j = 0;j <7; j++)
            //    {
            //        rssId += c[id % 36];
            //        id /= 36;
            //    }
            //    var temp = rssId.ToCharArray();
            //    Array.Reverse(temp);
            //    rssId = new string(temp);
            //    await processRSS("https://anchor.fm/s/" + rssId + "/podcast/rss");
            //}
            await scrapeTopEpsIndoSpotify();
        }

        public async Task getRssIndoFromChart()
        {
            var client = _httpClientFactory.CreateClient();
            var rssIds = new List<string>();
            try
            {
                string responseBody = await client.GetStringAsync(chartPodcastIDUrl);
                JsonArray jNode = JsonNode.Parse(responseBody)!.AsArray();
                for(int i=10; i<100; i++)
                {
                    string showName = jNode[i]!["showName"]!.ToString();
                    if (showName == "Rintik Sedu") continue;
                    string rssId = await getRssIdFromShowName(showName);
                    if(rssId != string.Empty)
                    {
                        rssIds.Add(rssId);
                    }
                }
            } catch (Exception ex) { Console.WriteLine(ex); }
            using (StreamWriter outputFile = new StreamWriter("./Scraping/rssID.txt"))
            {
                await outputFile.WriteAsync(String.Join("\n",rssIds));
            }
        }

        public async Task<string> getRssIdFromShowName(String showName)
        {
            string res = string.Empty;
            string temp_API_KEY = _config["CUSTOM_SEARCH_API_KEY"]!;
            string temp_cx = _config["SEARCH_ENGINE_ID"]!;

            var customSearchService = new CustomSearchAPIService(new BaseClientService.Initializer() {
                ApiKey = temp_API_KEY
            });

            var cseListRequest = customSearchService.Cse.List();
            cseListRequest.Cx = temp_cx;
            cseListRequest.ExactTerms = "rss";
            cseListRequest.Q = "anchor.fm " + showName;
            var cseListResponse = await cseListRequest.ExecuteAsync();
            if(cseListResponse.Items != null && cseListResponse.Items.Count>0)
            {
                var item = cseListResponse.Items[0];
                Console.WriteLine(item.Link);
                if(item.Link.Contains("anchor.fm"))
                {
                    res = await processRssIfValid(item.Link, showName);
                }
            }

            return res;
        }

        public async Task<string> processRssIfValid(string rssUrl, string showName)
        {
            string res = string.Empty;
            var client = _httpClientFactory.CreateClient();
            try
            {
                Stream stream = await client.GetStreamAsync(rssUrl);
                XmlReader reader = XmlReader.Create(stream);
                SyndicationFeed feed = SyndicationFeed.Load(reader);
                string podcastName = feed.Title.Text;
                podcastName = Util.ConvertToPlainText(podcastName).Replace("\r", "").Replace("\n", "").Trim();
                showName = Util.ConvertToPlainText(showName).Replace("\r", "").Replace("\n", "").Trim();
                if(showName==podcastName)
                {
                    Console.WriteLine(showName+ " sesuai dengan "+ podcastName);
                    res = rssUrl;
                } else
                {
                    Console.WriteLine(showName+ " berbeda dengan "+ podcastName);
                }
            } catch(Exception ex) { Console.WriteLine(ex); }
            return res;
        }

        public async Task processRSS(string rss_url)
        {
            var scrapingRSS = new ScrapingRSS(_db, _httpClientFactory);
			try
			{
				SyndicationFeed feed = await scrapingRSS.getSyndicationFeed(rss_url);
				if (!scrapingRSS.isAllowedFeed(feed))
				{
					return;
				}
				(string podcastName, string defaultAuthor, string defaultImageUrl, HashSet<int> categoriesId) = scrapingRSS.getPodcastInfo(feed);

			    try
                {
                    SyndicationItem lastEp = feed.Items.First();
                    Audio newAudio = scrapingRSS.getEpsAudio(lastEp, podcastName, defaultAuthor, defaultImageUrl);
                    newAudio.platform = "Anchor";
                    if(await scrapingRSS.isValidUrl(newAudio.platformUrl)) {
                        if(!FilterInappropriateContent.isContainBadWords(newAudio.title))
                        {
                            scrapingRSS.processAudioToDb(newAudio, categoriesId);
                        }
                    }
                } catch (Exception ex) { Console.WriteLine(ex); }
                try
                {
                    SyndicationItem firstEp = feed.Items.Last();
					Audio newAudio = scrapingRSS.getEpsAudio(firstEp, podcastName, defaultAuthor, defaultImageUrl);
					newAudio.platform = "Anchor";
					if (await scrapingRSS.isValidUrl(newAudio.platformUrl)) {
						if (!FilterInappropriateContent.isContainBadWords(newAudio.title))
						{
							scrapingRSS.processAudioToDb(newAudio, categoriesId);
						}
					}
				} catch (Exception ex) { Console.WriteLine(ex); }
			} catch (Exception ex) { Console.WriteLine(ex); }
        }

        public async Task scrapeTopEpsIndoSpotify()
        {
            var client = _httpClientFactory.CreateClient();
            var config = SpotifyClientConfig.CreateDefault().
                WithHTTPLogger(new SimpleConsoleHTTPLogger());
            var spotify = new SpotifyClient(config.WithToken(spotifyAccessToken));
            var CLIENT_ID = _config["SPOTIFY_CLIENT_ID"]!;
            var CLIENT_SECRET = _config["SPOTIFY_CLIENT_SECRET"]!;
			try
			{
                string responseBody = await client.GetStringAsync(chartEpsIDUrl);
                JsonArray jNode = JsonNode.Parse(responseBody)!.AsArray();
                Console.WriteLine("jNode.Count: "+ jNode.Count);
                for(int i = 0; i < (jNode.Count+49)/50;i++)
                {
                    List<Audio> newAudios = new List<Audio>();
                    List<string> newAudioSpotifyId = new List<string>();
                    List<string> showUrls = new List<string>();
                    for(int j = i*50;j < Math.Min(jNode.Count,(i+1)*50); j++)
                    {
                        var newAudio = new Audio();
                        newAudio.title = jNode[j]!["showName"]!.ToString() + " / " + jNode[j]!["episodeName"]!.ToString();
                        newAudio.title = Util.ConvertToPlainText(newAudio.title).Replace("\r", "").Replace("\n", "").Trim();
                        newAudio.imageUrl = jNode[j]!["episodeImageUrl"]!.ToString();
                        newAudio.contentCreator = jNode[j]!["showPublisher"]!.ToString();
                        newAudio.platform = "Spotify";
                        newAudios.Add(newAudio);
                        newAudioSpotifyId.Add(getIdFromUri(jNode[j]!["episodeUri"]!.ToString()));
                        //Console.WriteLine(newAudioSpotifyId.Last());
                        showUrls.Add(getShowUrlFromUri(jNode[j]!["showUri"]!.ToString()));
                    }
                    bool tryToken = true;
                    EpisodesResponse? audiosResponse = null;
                    while (tryToken)
                    {
                        try
                        {
                            audiosResponse = await spotify.Episodes.GetSeveral(new EpisodesRequest(newAudioSpotifyId)
                            {
                                Market = "ID"
                            }) ;
                            tryToken = false;
                            Console.WriteLine("Dapat response");
                            //Console.WriteLine(audiosResponse.Episodes[0].ToString());
                        } catch(APIException e)
                        {
                            if(e.Response?.StatusCode.ToString() == "Unauthorized")
                            {
                                var request = new ClientCredentialsRequest(CLIENT_ID, CLIENT_SECRET);
                                var response = await new OAuthClient(config).RequestToken(request);
                                Console.WriteLine("Ulangi request dengan token " + response.AccessToken);
                                spotifyAccessToken = response.AccessToken;
                                spotify = new SpotifyClient(config.WithToken(response.AccessToken));
                            } else
                            {
                                tryToken= false;
                            }
                        }
                    }
                    for (int j = 0; j < newAudios.Count; j++)
                    {
                        Console.WriteLine("Episode title: " + audiosResponse!.Episodes[j].Name);
                        var fullEps = audiosResponse!.Episodes[j];
                        newAudios[j].description = fullEps.Description;
                        newAudios[j].url = fullEps.Uri;
                        if (Util.isAudioPathExist(_db, newAudios[j].url) || FilterInappropriateContent.isContainBadWords(newAudios[j].title))
                        {
                            continue;
                        }
                        newAudios[j].platformUrl = fullEps.ExternalUrls["spotify"];
                        newAudios[j].duration = fullEps.DurationMs / 1000;
                        if(fullEps.ReleaseDatePrecision == "day")
                        {
                            newAudios[j].dateUploaded = DateOnly.ParseExact(fullEps.ReleaseDate, "yyyy-MM-dd");   
                        } else if(fullEps.ReleaseDatePrecision == "Month")
                        {
                            newAudios[j].dateUploaded = DateOnly.ParseExact(fullEps.ReleaseDate, "yyyy-MM");
                        } else if(fullEps.ReleaseDatePrecision == "year")
                        {
                            newAudios[j].dateUploaded = DateOnly.ParseExact(fullEps.ReleaseDate, "yyyy");
                        }
                        HashSet<int> categoriesId = scrapeCategoriesFromShow(showUrls[j]);
                        _db.AudioList.Add(newAudios[j]);
                        _db.SaveChanges();
                        var epsId = _db.AudioList.Where(u => u.url == newAudios[j].url).Select(u => u.audioId).ToList()[0];
                        foreach (var categoryId in categoriesId)
                        {
                            //Console.WriteLine(categoryId);
                            var newAudioCategoryRelation = new AudioCategoryRelation();
                            newAudioCategoryRelation.audioId = epsId;
                            newAudioCategoryRelation.categoryId = categoryId;
                            _db.AudioCategoryRelation.Add(newAudioCategoryRelation);
                        }
                        _db.SaveChanges();
                    }
                }
            }
            catch (Exception ex) { Console.WriteLine(ex); }
        }

        private HashSet<int> scrapeCategoriesFromShow(string v)
        {
            HashSet<int> categoriesId = new HashSet<int>();
            HtmlWeb web = new HtmlWeb();
            var showPage = web.Load(v);
            var categoryNodes = showPage.DocumentNode.SelectNodes("//span[@class=\"ChipInner__ChipInnerComponent-sc-1ly6j4j-0 bhATCW\"]");
            if(categoryNodes != null)
            {
                foreach(var categoryNode in categoryNodes)
                {
                    Console.WriteLine(categoryNode.InnerText);
                    int categoryId = Util.findAndUpdateCategoryID(_db, categoryNode.InnerText);
                    categoriesId.Add(categoryId);
                }
            }
            return categoriesId;
        }

        private string getIdFromUri(string v)
        {
            for(int i = v.Length-1;i>=0;i--)
            {
                if (v.ElementAt(i-1) == ':')
                {
                    return v.Substring(i);
                }
            }
            return string.Empty;
        }

        private string getShowUrlFromUri(string v)
        {
            return "https://open.spotify.com/show/" + getIdFromUri(v);
        }
    }

}

