﻿using HtmlAgilityPack;
using System;
using System.Xml;
using System.ServiceModel.Syndication;
using Bharabas.Models;
using Bharabas.Data;
using Bharabas.Scraping;

namespace Bharabas.Scraping
{
	public class ScrapingGooglePodcast
	{
		const string baseUrl = "https://podcast.google.com/";
		private int count = 0;
		private readonly IHttpClientFactory _httpClientFactory;
		private readonly ApplicationDbContext _db;
		public ScrapingGooglePodcast(ApplicationDbContext db, IHttpClientFactory httpClientFactory)
		{
			_httpClientFactory = httpClientFactory;
			_db = db;
		}

		public async Task scrapeDataAsync()
		{
			Dictionary<string,string> platformUrlOfRss = new Dictionary<string,string>();
			List<string> rss_urls = getRssUrlsFromExplore(baseUrl, ref platformUrlOfRss);
			foreach(string rss_url in rss_urls)
			{
				try
				{
					await processRSS(rss_url, platformUrlOfRss[rss_url]);
				} catch (Exception ex)
				{
					Console.WriteLine(ex);
				}
			}
			Console.WriteLine("Finish Google Podcast");
		}

		public List<string> getRssUrlsFromExplore(string url, ref Dictionary<string,string> platformUrlFromRss)
		{
			List<string> rss_urls = new List<string>();
			HtmlWeb web = new HtmlWeb();
			var explorePage = web.Load(baseUrl);
			var podcastUrlNodes = explorePage.DocumentNode.SelectNodes("//c-wiz/div[1]/div[2]/c-wiz[2]/div[1]/div/scrolling-carousel/div[1]/span/div/a");
			foreach (var podcastUrlNode in podcastUrlNodes)
			{
				string rss_url = getRssUrlFromPodcast(podcastUrlNode.Attributes["href"].Value);
				platformUrlFromRss[rss_url] = podcastUrlNode.Attributes["href"].Value;
				//Console.WriteLine(rss_url);
				rss_urls.Add(rss_url);
				//if (rss_urls.Count() >= 10) break;//TODO: unkomen kalo udah testing 
			}
			return rss_urls;
		}

		public string getRssUrlFromPodcast(string podcastUrl)
		{
			HtmlWeb web = new HtmlWeb();
			//Console.WriteLine(baseUrl+podcastUrl);
			var podcastPage = web.Load(baseUrl+podcastUrl);
			string rssComp = podcastPage.DocumentNode.SelectSingleNode("//c-wiz/div[1]/div[2]/div[1]/div[1]/div[1]/c-wiz/c-data").Attributes["jsdata"].Value;
			string[] calonRss = rssComp.Split(';');
			for(int i= 0;i < calonRss.Length;i++)
			{
				if (calonRss[i].StartsWith("http"))
				{
					return calonRss[i];
				}
			}
			return "Rss not found";
		}

		public async Task processRSS(string rss_url, string platformUrl)
		{
			var scrapingRSS = new ScrapingRSS(_db, _httpClientFactory);
			try
			{
				SyndicationFeed feed = await scrapingRSS.getSyndicationFeed(rss_url);
				if (!scrapingRSS.isAllowedFeed(feed))
				{
					return;
				}
				(string podcastName, string defaultAuthor, string defaultImageUrl, HashSet<int> categoriesId) = scrapingRSS.getPodcastInfo(feed);

				HtmlWeb web = new HtmlWeb();
				var podcastPage = web.Load(baseUrl+platformUrl);
				var epNodes = podcastPage.DocumentNode.SelectNodes("//c-wiz/div[1]/div[2]/div[1]/div[2]/div[2]/a");
				try
				{
					SyndicationItem? lastEp = null;
					string lastEpUrl = baseUrl + epNodes.First().Attributes["href"].Value;
					string lastEpTitle = epNodes.First().SelectSingleNode("./div[1]/div[2]/div[1]").InnerText;
					lastEpTitle = Util.ConvertToPlainText(lastEpTitle).Replace("\r", "").Replace("\n", "").Trim();
					for(int i = 0; i < feed.Items.Count(); i++)
					{
						if(Util.ConvertToPlainText(feed.Items.ElementAt(i).Title.Text).Replace("\r","").Replace("\n","").Trim() == lastEpTitle)
						{
							lastEp = feed.Items.ElementAt(i);
							Audio newAudio = scrapingRSS.getEpsAudio(lastEp, podcastName, defaultAuthor, defaultImageUrl);
							newAudio.platformUrl = lastEpUrl;
							newAudio.platform = "Google Podcast";
							if(!FilterInappropriateContent.isContainBadWords(newAudio.title))
							{
								scrapingRSS.processAudioToDb(newAudio, categoriesId);
							}
							break;
						}
					}
					if(lastEp == null ) 
					{
						Console.WriteLine("Platform Url: " + baseUrl + platformUrl);
						Console.WriteLine("lastEpTitle: " + lastEpTitle);
					}
				} catch(Exception ex) { Console.WriteLine(ex); }
				try
				{
					SyndicationItem? firstEp = null;
					string firstEpUrl = baseUrl + epNodes.Last().Attributes["href"].Value;
					string firstEpTitle = epNodes.Last().SelectSingleNode("./div[1]/div[2]/div[1]").InnerText;
					firstEpTitle = Util.ConvertToPlainText(firstEpTitle).Replace("\r", "").Replace("\n", "").Trim();
					for(int i = feed.Items.Count() - 1; i >= 0; i--)
					{
						if(Util.ConvertToPlainText(feed.Items.ElementAt(i).Title.Text).Replace("\r","").Replace("\n","").Trim() == firstEpTitle) 
						{
							firstEp = feed.Items.ElementAt(i);
							Audio newAudio = scrapingRSS.getEpsAudio(firstEp, podcastName, defaultAuthor, defaultImageUrl);
							newAudio.platformUrl = firstEpUrl;
							newAudio.platform = "Google Podcast";
							if (!FilterInappropriateContent.isContainBadWords(newAudio.title))
							{
								scrapingRSS.processAudioToDb(newAudio, categoriesId);
							}
							break;
						}
					}
					if(firstEp == null)
					{
						Console.WriteLine("Platform Url: " + baseUrl + platformUrl);
						Console.WriteLine("firstEpTitle: " + firstEpTitle);
					}
				} catch(Exception ex) { Console.WriteLine(ex);}
			} catch (Exception ex) { Console.WriteLine(ex); }
		}
	}
}
