﻿using Microsoft.AspNetCore.Authentication;
using HtmlAgilityPack;
using System.ComponentModel.DataAnnotations;
using Bharabas.Data;
using Bharabas.Models;
using System.Text.RegularExpressions;

namespace Bharabas.Scraping
{
    public class ScrapingListenNotes
    {
        private readonly ApplicationDbContext _db;
        const string baseUrl = "https://www.listennotes.com/best-podcasts/";
        public ScrapingListenNotes(ApplicationDbContext db)
        {
            _db = db;
            
            //this.scrapeData();
        }

        public void scrapeData()
        {
            HtmlWeb web =  new HtmlWeb();
            List<string>categoryUrls = new List<string>();
            List<string> categoryUrlNames = new List<string>();

            var categoryDoc = web.Load(baseUrl);
            var categoryNodes = categoryDoc.DocumentNode.SelectNodes("/html/body/div[1]/div[3]/div[2]/div[1]/div[2]/div[2]/div[2]/div");
            foreach (var categoryNode in categoryNodes)
            {
                var categoryStr = categoryNode.SelectSingleNode("a").InnerText.Replace("amp;", "").Replace("\n", "");

                findAndUpdateCategoryID(categoryStr);

                categoryUrls.Add(categoryNode.SelectSingleNode("a").Attributes["href"].Value);
                categoryUrls.Add(categoryNode.SelectSingleNode("a").Attributes["href"].Value + "&page=2");
                categoryUrlNames.Add(categoryStr);
                categoryUrlNames.Add(categoryStr);
            }

            var i = 0;
            foreach(var categoryUrl in categoryUrls)
            {
                var doc = web.Load(categoryUrl);
                var nodes = doc.DocumentNode.SelectNodes("/html/body/div[1]/div[3]/div[2]/div[2]/div[2]/div/div[2]/div[1]/div/div[position()>2]");
                foreach(var node in nodes)
                {
                    if (node.Attributes["class"] is null) continue;
                    if (!this.isClassValid(node.Attributes["class"].Value)) continue;

                    try
                    {
                        // duration string
                        var durationStr = node.SelectSingleNode("div/div[4]/div[2]/div/div[2]/div/div[2]").InnerText;
                        durationStr = this.formatStr(durationStr);
                        durationStr = durationStr.Replace(" ", "");
                        if (durationStr == "0") continue;
                        var duration = this.toSecond(durationStr);

                        // title
                        var titleAlbum = node.SelectSingleNode("div/div[1]/h2/a").Attributes["title"].Value;
                        var titleEpisode = node.SelectSingleNode("div/div[4]/div[2]/div/div[1]/div/h3/a").Attributes["title"].Value;
                        var title = titleAlbum + " / " + titleEpisode;
                        title = this.formatStr(title);

                       // description
                       //var description = node.SelectSingleNode("div/div[3]").InnerText;
                       // description = this.formatStr(description);

                        // audio_url
                        var audio = node.SelectSingleNode("div/div[4]/div[2]/div/div[2]").Attributes["data-audio"].Value;

                        // image_url
                        var img = node.SelectSingleNode("div/div[2]/div[1]/div[1]/a").Attributes["href"].Value;

                        // date
                        var datePod = "";
                        var dateRaw = node.SelectSingleNode("div/div[4]/div[2]/div/div[1]/div/div/time").InnerText.Trim();
                        if (dateRaw == "today")
                        {
                            datePod = DateTime.Today.ToString("yyyy-MM-dd");
                        }
                        else if (dateRaw == "yesterday")
                        {
                            datePod = DateTime.Today.AddDays(-1).ToString("yyyy-MM-dd");
                        }
                        else
                        {
                            datePod = this.parseDate(dateRaw);
                        }

                        // platform url
                        var platformUrl = node.SelectSingleNode("div/div[1]/h2/a").Attributes["href"].Value;

                        var detailDoc = web.Load(platformUrl);
                        var description = detailDoc.DocumentNode.SelectSingleNode("/html/body/div[1]/div[3]/div[2]/div[2]/div[2]/div/div[2]/div[1]/div/div[3]/div[1]").InnerHtml;
                        description = this.formatStr(description);

                        // creator
                        var creator = node.SelectSingleNode("div/div[2]/div[1]/div[3]/div[1]/a").InnerText;
                        creator = this.formatStr(creator);

                        // check if content already exists
                        if (this.isAudioPathExist(audio)) continue;

                        Console.WriteLine(description);

                        // update table audio
                        var newObjAudio = new Audio();
                        newObjAudio.title = Regex.Replace(title!, @"[^\u0000-\u007F]+", string.Empty)!;
                        newObjAudio.description = Regex.Replace(description!, @"[^\u0000-\u007F]+", string.Empty)!;
                        newObjAudio.url = audio!;
                        newObjAudio.imageUrl = img!;
                        newObjAudio.platformUrl = platformUrl;
                        newObjAudio.contentCreator = creator!;
                        newObjAudio.platform = "ListenNotes";
                        newObjAudio.duration = duration!;
                        newObjAudio.dateUploaded = DateOnly.Parse(datePod!);
                        newObjAudio.likesCount = 0;

                        _db.AudioList.Add(newObjAudio);
                        _db.SaveChanges();

                        var audioID = _db.AudioList.Where(u => u.url == audio).Select(u => u.audioId).ToList();

                        var newAudioCategoryRelationObj = new AudioCategoryRelation();
                        newAudioCategoryRelationObj.audioId = audioID[0]!;
                        newAudioCategoryRelationObj.categoryId = findAndUpdateCategoryID(categoryUrlNames[i]!);

                        _db.AudioCategoryRelation.Add(newAudioCategoryRelationObj);
                        _db.SaveChanges();
                    }
                    catch
                    {

                    }
                    

                }
                i++;
            }
			var filter = new FilterInappropriateContent(_db);
			filter.Filter();
			Console.WriteLine("finish");

        }

        public bool isClassValid(string cls)
        {
            return cls == "ln-page-card";
        }

        public string formatStr(string str)
        {
            return str.Replace("amp;", "").Replace("&#x27;", "\'").Replace("&nbsp;"," ").Replace("\n"," ").Replace("<br>","\n");
        }

        public int toSecond(string str)
        {
            var ret = 0;
            ret += (str[0] - '0') * 3600 * 10;
            ret += (str[1] - '0') * 3600;
            ret += (str[3] - '0') * 60 * 10;
            ret += (str[4] - '0') * 60;
            ret += (str[6] - '0') * 10;
            ret += (str[7] - '0');

            return ret;
        }

        public string parseDate(string str)
        {
            List<string> listDate = new List<string> { "Jan", "Feb", "March", "April", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec" };
            char[] delimiterChars = { ' ' };

            string[] words = str.Split(delimiterChars);
            string ret = "";
            for(int i = 0; i < words.Length; i++)
            {
                words[i] = this.formatStr(words[i].Replace(",", "").Replace(".",""));
            }
            ret += words[2].ToString();
            ret += "-";
            for(int i = 0; i < 12; i++)
            {
                if (listDate[i] == words[0])
                {
                    if(i+1 < 10)
                    {
                        ret += "0";
                    }
                    ret += (i+1).ToString();
                }
            }
            ret += "-";
            if (words[1].ToString().Length == 1)
            {
                ret += "0";
            }
            ret += words[1].ToString();

            return ret;
        }

        public int findAndUpdateCategoryID(string categoryName)
        {
            var categoryID = _db.Category.Where(u => u.name == categoryName).Select(u => u.categoryId).ToList();

            if(categoryID.Count == 0) 
            {
                Category objCategory = new Category();
                objCategory.name = categoryName;
                _db.Category.Add(objCategory);
                _db.SaveChanges();

                categoryID = _db.Category.Where(u => u.name == categoryName).Select(u => u.categoryId).ToList();
            }
            Console.WriteLine(categoryID[0]);
            return categoryID[0];
        }

        public bool isAudioPathExist(string path)
        {
            var audio = _db.AudioList.Where(u => u.url == path).Select(u => u.url).ToList();
            return audio.Count != 0;
        }
    }
}
