﻿using HtmlAgilityPack;
using System;
using System.Xml;
using System.ServiceModel.Syndication;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using Bharabas.Models;
using Bharabas.Data;
using Bharabas.Scraping;
using System.Xml.Linq;
using Google.Apis.YouTube;
using Google.Apis.Auth.OAuth2;
using Google.Apis.Services;
using Google.Apis.Upload;
using Google.Apis.Util.Store;
using Google.Apis.YouTube.v3;
using Google.Apis.YouTube.v3.Data;



namespace Bharabas.Scraping
{
    public class ScrapingYouTube
    {
        private readonly ApplicationDbContext _db;
        private readonly IConfiguration _config;
        private List<string> categoryList = new List<string> { "Arts", "Business", "Comedy", "Education", "Fiction", "Government", "Health & Fitness", "History", "Kids & Family", "News", "Science", "Society & Culture", "Sports", "Technology" };

        public ScrapingYouTube(ApplicationDbContext db, IConfiguration config)
        {
            _db = db;
            _config = config;
            //foreach (var category in categoryNeeded)
            //{
            //    FindAndUpdateCategoryID(category);
            //}
            //this.ScrapeData().Wait();
        }

        public async Task ScrapeData()
        {
            var youtubeService = new YouTubeService(new BaseClientService.Initializer()
            {
                ApiKey = _config["GOOGLE_API_KEY"],
                ApplicationName = "Bharabas"
            });

            var nextPageToken = "";
            var pages = 2;
            var indexCategory = categoryList.Count;

            for (var i = 0; i < indexCategory; i++)
            {
                var categoryId = findAndUpdateCategoryID(categoryList[i]);
                for (var j = 0; j < pages; j++)
                {
                    var searchListRequest = youtubeService.Search.List("snippet");
                    searchListRequest.Q = categoryList[i] + " Podcast"; // Replace with your search term.
                    searchListRequest.MaxResults = 100;
                    if (nextPageToken != "")
                    {
                        searchListRequest.PageToken = nextPageToken;
                    }

                    // Call the search.list method to retrieve results matching the specified query term.
                    var searchListResponse = await searchListRequest.ExecuteAsync();

                    List<string> videos = new List<string>();
                    List<string> channels = new List<string>();
                    List<string> playlists = new List<string>();

                    // Add each result to the appropriate list, and then display the lists of
                    // matching videos, channels, and playlists.

                    foreach (var searchResult in searchListResponse.Items)
                    {
                        if (searchResult.Id.Kind == "youtube#video")
                        {
                            var title = searchResult.Snippet.Title;

                            var description = searchResult.Snippet.Description;

                            var image_url = searchResult.Snippet.Thumbnails.High.Url;

                            DateTime dateTime;
                            if (searchResult.Snippet.PublishedAt != null)
                            {
                                dateTime = (DateTime) searchResult.Snippet.PublishedAt;
                            }
                            else
                            {
                                dateTime = DateTime.Now;
                            }
                            var date = DateOnly.FromDateTime(dateTime);


                            var creator = searchResult.Snippet.ChannelTitle;

                            var id = searchResult.Id.VideoId;
                            if(isAudioPathExist(id))
                            {
                                continue;
                            }

                            var video_url = "https://www.youtube.com/watch?v=" + searchResult.Id.VideoId;

                            var category = "";

                            int duration = 0;

                            var videoRequest = youtubeService.Videos.List("contentDetails,snippet");
                            videoRequest.Id = searchResult.Id.VideoId;

                            var videoResponse = await videoRequest.ExecuteAsync();

                            foreach(var video in videoResponse.Items)
                            {
                                description = video.Snippet.Description;

                                var youtubeDuration = video.ContentDetails.Duration;
                                duration = Convert.ToInt32(XmlConvert.ToTimeSpan(youtubeDuration).TotalSeconds.ToString());
                        
                                Console.WriteLine(duration);
                                Console.WriteLine(category);
                            }

                            Console.WriteLine(title);
                            Console.WriteLine(description);
                            Console.WriteLine(image_url);
                            Console.WriteLine(date);
                            Console.WriteLine(creator);
                            Console.WriteLine(video_url);
                            Console.WriteLine("\n");

                            title = Util.ConvertToPlainText(title).Replace("\r", "").Replace("\n", "").Trim();
                            description = Util.ConvertToPlainText(description).Replace("\r", "").Replace("\n", "").Trim();

							var newObjAudio = new Audio();
                            newObjAudio.title = title;
                            newObjAudio.description = description;
                            newObjAudio.imageUrl= image_url;
                            newObjAudio.contentCreator = creator;
                            newObjAudio.platform = "YouTube";
                            newObjAudio.duration = duration;
                            newObjAudio.dateUploaded = date;
                            newObjAudio.likesCount = 0;
                            newObjAudio.url = id;
                            newObjAudio.platformUrl = video_url;

                            try
                            {
                                _db.AudioList.Add(newObjAudio);
                                _db.SaveChanges();

                                var audioID = _db.AudioList.Where(u => u.url == id).Select(u => u.audioId).ToList();

                                var newAudioCategoryRelationObj = new AudioCategoryRelation();
                                newAudioCategoryRelationObj.audioId = audioID[0]!;
                                newAudioCategoryRelationObj.categoryId = categoryId;

                                _db.AudioCategoryRelation.Add(newAudioCategoryRelationObj);
                                _db.SaveChanges();
                            }catch (Exception ex)
                            {
                                Console.WriteLine(ex.ToString());
                            }
                            
                            
						}
                    }

                    nextPageToken = searchListResponse.NextPageToken;
                }
                nextPageToken = "";
            }


			//HtmlWeb web = new HtmlWeb();
			//HtmlWeb videoWeb = new HtmlWeb();
			//List<string> categoryUrls = new List<string>();
			//List<string> categoryUrlNames = new List<string>();
			//var i = 0;

			//foreach (var category in categoryList)
			//{
			//    var categoryDoc = web.Load(baseUrl + "/results?search_query=podcast+" + category);
			//    var categoryNodes = categoryDoc.DocumentNode.SelectNodes("//div[@class='style-scope ytd-video-renderer']");

			//    foreach (var categoryNode in categoryNodes)
			//    {
			//        var videoHref = categoryNode.SelectSingleNode("ytd-thumbnail").SelectSingleNode("a").Attributes["href"].Value;
			//        var videoDoc = videoWeb.Load(baseUrl + videoHref);
			//        var infoNodes = videoDoc.DocumentNode.SelectSingleNode("style-scope ytd-player-microformat-renderer");

			//        // duration string 
			//        var durationStr = categoryNode.SelectSingleNode("ytp-time-duration").InnerText;
			//        var duration = this.toSecond(durationStr);

			//        // title
			//        var title = infoNodes.Attributes["name"].Value;

			//        // description
			//        var description = infoNodes.Attributes["description"].Value;

			//        // audio_url ...
			//        var embedUrl = infoNodes.Attributes["embedUrl"].Value;

			//        // image_url
			//        var img = infoNodes.Attributes["thumbnailUrl"].Value;

			//        // date
			//        var date = infoNodes.Attributes["uploadDate"].Value;

			//        // creator
			//        var creator = infoNodes.Attributes["author"].Value;


			//        var newObjAudio = new Audio();
			//        newObjAudio.title = title!;
			//        newObjAudio.description = description!;
			//        newObjAudio.url = embedUrl!;
			//        newObjAudio.imageUrl = img!;
			//        newObjAudio.contentCreator = creator!;
			//        newObjAudio.platform = "YouTube";
			//        newObjAudio.duration = duration!;
			//        newObjAudio.dateUploaded = DateOnly.Parse(date!);
			//        newObjAudio.likesCount = 0;

			//        _db.AudioList.Add(newObjAudio);
			//        _db.SaveChanges();

			//        var audioID = _db.AudioList.Where(u => u.url == embedUrl).Select(u => u.audioId).ToList();

			//        var newAudioCategoryRelationObj = new AudioCategoryRelation();
			//        newAudioCategoryRelationObj.audioId = audioID[0]!;

			//        var categoryID = _db.Category.Where(u => u.name == category).Select(u => u.categoryId).ToList();

			//        if (categoryID.Count == 0)
			//        {
			//            Category objCategory = new Category();
			//            objCategory.name = category;
			//            _db.Category.Add(objCategory);
			//            _db.SaveChanges();

			//            categoryID = _db.Category.Where(u => u.name == category).Select(u => u.categoryId).ToList();
			//        }

			//        newAudioCategoryRelationObj.categoryId = categoryID[0]!;

			//        _db.AudioCategoryRelation.Add(newAudioCategoryRelationObj);
			//        _db.SaveChanges();
			//    }
			//}

			var filter = new FilterInappropriateContent(_db);
			filter.Filter();
		}


		public int findAndUpdateCategoryID(string categoryName)
		{
			var categoryID = _db.Category.Where(u => u.name == categoryName).Select(u => u.categoryId).ToList();

			if (categoryID.Count == 0)
			{
				Category objCategory = new Category();
				objCategory.name = categoryName;
				_db.Category.Add(objCategory);
				_db.SaveChanges();

				categoryID = _db.Category.Where(u => u.name == categoryName).Select(u => u.categoryId).ToList();
			}
			Console.WriteLine(categoryID[0]);
			return categoryID[0];
		}

		public bool isAudioPathExist(string path)
		{
			var audio = _db.AudioList.Where(u => u.url == path).Select(u => u.url).ToList();
			return audio.Count != 0;
		}
	}
}
