using Bharabas.Controllers;
using Bharabas.Data;
using Bharabas.Models;
using Bharabas.Scraping;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Xunit.Abstractions;
using Xunit.Sdk;

namespace BharabasTest
{
    public class UnitTest
    {
        private readonly ILogger<HomeController> _logger;
        private readonly ApplicationDbContext _db;
        private readonly IHttpClientFactory _httpClientFactory;
        public IConfigurationRoot Configuration { get; set; }
        private readonly ITestOutputHelper _testOutputHelper;
        public UnitTest(ITestOutputHelper testOutputHelper) {
          
            var builder = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json", optional: false, reloadOnChange: true);
            Configuration = builder.Build();

            var optionsBuilder = new DbContextOptionsBuilder<ApplicationDbContext>().UseNpgsql(Configuration.GetConnectionString("DefaultConnection"));
            _db = new ApplicationDbContext(optionsBuilder.Options);
            _testOutputHelper = testOutputHelper;

        }
        [Fact]
        public void TestHomeSearch()
        {
            var homeController = new HomeController(_db);
            Assert.Equal(homeController.getSearchResult("","",1).Count(),_db.AudioList.Count());
        }
        [Fact]
        public void TestScraping()
        {
            var scrapingListenNotes = new ScrapingListenNotes(_db);
            Assert.Equal(scrapingListenNotes.parseDate("April 18, 2023"), "2023-04-18");
        }

        [Fact]
        public void TestNavCategory()
        {
            var sharedController = new SharedController(_db);
            Assert.True(sharedController.getListCategory().Count() > 5);
        }

        [Fact]
        public void TestPager()
        {
            var pager = new Pager(60,9,6);
            Assert.Equal(pager.EndPage, 10);
        }
    }
}